package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.TeaGardenPlotDao;
import com.xkrs.microservice.model.entity.TeaGardenPlotEntity;
import com.xkrs.microservice.model.qo.TeaGardenPlotQo;
import com.xkrs.microservice.service.TeaGardenPlotService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

import static com.xkrs.microservice.util.OpenGeoUtil.wktStrToGeom;


/**
 * 茶园地块服务管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "TeaGardenPlotServiceCache")
public class TeaGardenPlotServiceImpl implements TeaGardenPlotService {

    Logger logger = LoggerFactory.getLogger(TeaGardenPlotServiceImpl.class);

    @Resource
    private TeaGardenPlotDao teaGardenPlotDao;

    /**
     * 获取所有典型地块记录
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public List<TeaGardenPlotEntity> getAllRecord(){
        return teaGardenPlotDao.findAll();
    }

    /**
     * 获取指定地块编号记录
     * @param dkbh
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public TeaGardenPlotEntity getAllRecordByDkbh(String dkbh){
        return teaGardenPlotDao.findAllByDkbh(dkbh);
    }


    /**
     * 新增典型地块记录
     * @param teaGardenPlotQo
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public void add(TeaGardenPlotQo teaGardenPlotQo){
        TeaGardenPlotEntity teaGardenPlotEntity = new TeaGardenPlotEntity();
        CopyPropertiesUtil.copy( teaGardenPlotQo, teaGardenPlotEntity);
        if( !teaGardenPlotQo.getGeomStr().isEmpty()){
            // 转化geometry
            teaGardenPlotEntity.setGeom(wktStrToGeom(teaGardenPlotQo.getGeomStr()));
        }
        teaGardenPlotDao.save(teaGardenPlotEntity);
    }

    /**
     * 更新典型地块记录
     * @param teaGardenPlotQo
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public void update(TeaGardenPlotQo teaGardenPlotQo){
        TeaGardenPlotEntity teaGardenPlotEntity = new TeaGardenPlotEntity();
        CopyPropertiesUtil.copy( teaGardenPlotQo, teaGardenPlotEntity);
        if( !teaGardenPlotQo.getGeomStr().isEmpty()){
            // 转化geometry
            teaGardenPlotEntity.setGeom(wktStrToGeom(teaGardenPlotQo.getGeomStr()));
        }
        teaGardenPlotDao.save(teaGardenPlotEntity);
    }

    /**
     * 删除典型地块记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public int delete(Integer id){
        Optional<TeaGardenPlotEntity> teaGardenPlotEntityOptional = teaGardenPlotDao.findById(id);
        if(teaGardenPlotEntityOptional.isPresent()){
            return 1;
        }
        teaGardenPlotDao.deleteById(id);
        return 0;
    }

}
